// ********************************************************************
// VNC1L/VDIP1 Vinculum - Grundfunktionen 
// 8051-Version / SDCC 2.9.0
// Stand: V1.24 / 16.11.2009
// Author: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//
// ***********************************************************************
// VNC1L - USB HOST CONTROLLER - SPI-MODE (MC: Master, VNC1L: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.68 getestet
// ***********************************************************************

// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// ***********************************************************************
void vnc_init(void) {
  CS_VNC_L;			// SPI-Interface inaktiv
  SCLK_L;			// Ruhezustand der SPI-Leitungen herstellen
  SDI_H;
  RE_VNC_L;			// Hardware-Reset VNC1L
  delay_ms(100);	// Pause
  RE_VNC_H;			// VNC1L starten
  delay_ms(500);	// Pause fr Anlauf
}

// ***********************************************************************
// VNC1L - 13-Bit-SPI Senden eines Zeichens
// SPI Mode
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: status    : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_spi(unsigned char wr_byte) {
unsigned char stelle, status, cnt;
  cnt = TIMEOUT_SPI;	// Anzahl der Schreib-Versuche bis Timeout
  do {
    CS_VNC_H;			// SPI-Interface aktiv
    SDI_H;				// Startbit (immer 1)
    SCLK_H;  SCLK_L;	// L/H Flanke
    SDI_L;				// RW-Bit (0=Write)
    SCLK_H;  SCLK_L;	// L/H Flanke
    SDI_L;				// ADDR (0=Daten)
    SCLK_H;  SCLK_L;	// L/H Flanke
    // 8 Datenbits senden (MSB zuerst)
    for ( stelle = 0x80; stelle; stelle >>= 1 ) {
      if (wr_byte & stelle) SDI_H; else SDI_L;
      SCLK_H;  SCLK_L;	// L/H Flanke
    }
    // Statusbit einlesen
    status = SDO_VNC;	// Statusbit (0=Accept, 1=Reject)
    SCLK_H;  SCLK_L;	// L/H Flanke
    // End Sequenz
    CS_VNC_L;			// SPI-Interface inaktiv
    SCLK_H;  SCLK_L;	// L/H Flanke
    // SDO    = 1;   	// nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
    cnt--;
	if (status) delay_us(150);	// Pause, wenn Byte nicht besttigt wurde
  } while (status && cnt);
#ifdef  USART_MONITORING
	usart_send_byte(wr_byte);	// gesendetes Zeichen seriell senden
#endif
#ifdef LCD_MONITORING
	lcd_monitor_byte(wr_byte);	// gesendetes Zeichen auf LCD anzeigen
#endif
  return status;
} 

// ***********************************************************************
// VNC1L - 13-Bit-SPI Empfangen eines Zeichens
// SPI Mode
// bergabe: rd_fkt    : Funktion (0=Datenbyte, 1=Statusbyte)
// Datenbyte wird im Buffer vnc-Stukturvariable gespeichert
// Rckgabe: status    : 0=Neues Byte im Buffer (ok) 1=kein neues Byte vom VNC1L vorhanden
// ***********************************************************************
unsigned char vnc_rd_spi(unsigned char rd_fkt) {
unsigned char stelle, daten, status;
unsigned int cnt;
  cnt = TIMEOUT_SPI;	//  Anzahl der Lese-Versuche bis Timeout
  do {
    daten  = 0;
    CS_VNC_H;			// SPI-Interface aktiv
    SDI_H;				// Startbit (immer 1)
    SCLK_H;  SCLK_L;	// L/H Flanke
    SDI_H;				// RW-Bit (1=Read)
    SCLK_H;  SCLK_L;	// L/H Flanke
    if (rd_fkt) SDI_H; else SDI_L;	// ADDR (0=Daten, 1=SPI-Statusinformation) 
    SCLK_H;  SCLK_L;	// L/H Flanke
    // 8 Datenbits empfangen (MSB zuerst)
    for ( stelle = 0x80; stelle; stelle >>= 1 ) {
      if (SDO_VNC) daten |= stelle;	// Datenbit speichern
      SCLK_H;  SCLK_L;	// L/H Flanke
    }
    // Statusbit einlesen
    status = SDO_VNC;	// Statusbit (0=New Data, 1=Old Data)
    SCLK_H;  SCLK_L;	// L/H Flanke
    // End Sequenz
    CS_VNC_L;			// SPI-Interface inaktiv
    SCLK_H;  SCLK_L;	// L/H Flanke
    // SDO    = 1;   	// nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
    if (!status) {
      // empfangenes Zeichen ist ein neues Zeichen
      // Empfangsbuffer um eine Stelle verschieben
      for ( stelle = VNC_BUFFER_SIZE - 1; stelle; stelle-- ) vnc.rec_buffer[stelle] = vnc.rec_buffer[stelle - 1];
      vnc.rec_buffer[0] = daten;	// neues Zeichen an 1. Stelle speichern
#ifdef  USART_MONITORING
		if (!rd_fkt) {
			usart_send_byte(vnc.rec_buffer[0]);		// empfangenes Zeichen seriell senden
		}
#endif
#ifdef LCD_MONITORING
		if (!rd_fkt) {
			lcd_monitor_byte(vnc.rec_buffer[0]);	// empfangenes Zeichen auf LCD anzeigen
		}
#endif
    }
    cnt--;
	if (status) delay_us(150);						 // Pause, wenn keine neues Byte vorliegt
  } while (status && cnt);
  return status;
} 

// ***********************************************************************
// VNC1L - Senden eines Zeichens
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: status    : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_byte(unsigned char wr_byte) {
unsigned char status;
unsigned int cnt;
  cnt = TIMEOUT_READ_WRITE;				// Anzahl der Sende-Versuche bis Timeout
  do {
    // VNC1L Status-Byte lesen
    if (vnc_rd_spi(1)) return 1;		// Fehler, Byte wurde nicht angenommen
	// Statusbyte liegt jetzt im Buffer
	status = vnc.rec_buffer[0] & 0x01;	// Test auf VNC1L RECEIVE-BUFFER voll (RXF# Bit0=1)
    cnt--;
	if (status) delay_us(150);			// Pause, wenn RECEIVE-BUFFER voll (RXF#)
  } while (status && cnt);
  if (status || (!cnt)) return 1;		// Fehler, Byte wurde nicht angenommen
  // VNC1L ist jetzt empfangsbereit
  if (vnc_wr_spi(wr_byte)) return 1;	// Fehler Byte wurde nicht angenommen
  return 0;	// Byte wurde gesendet
}

// ***********************************************************************
// VNC1L - Empfangen eines Zeichens
// Rckgabe: status    : 0=Neues Byte im Buffer (ok) 1=kein neues Byte vom VNC1L vorhanden
// ***********************************************************************
unsigned char vnc_rd_byte(void) {
unsigned char status;
unsigned int cnt;
  cnt = TIMEOUT_READ_WRITE;			// Anzahl der Empfangs-Versuche bis Timeout
  do {
    // VNC1L Status-Byte lesen
    status = vnc_rd_spi(0);			// Wenn Status=1 liegt kein neues Byte vor
	// Prfung auf TXE# im Status-Byte brachte keinen Vorteil
    cnt--;
	if (status) delay_us(150);		// Pause, wenn kein neue Byte vorliegt
  } while (status && cnt);
  if (status || (!cnt)) return 1;	// Fehler, kein Byte empfangen
  return 0;	// Byte wurde empfangen
}

// ***********************************************************************
// VNC1L - Senden eines Kommandos
// bergabe: wr_cmd : Kommando (Adresse)
// Rckgabe: status    : 0=ok 1=Kommando nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_cmd(char *wr_cmd) {
unsigned char vnc_i;	// vnc_offset;
  for(vnc_i = strlen(wr_cmd); vnc_i; vnc_i--) {
    if (vnc_wr_byte(*wr_cmd)) return 1;	// Fehler, berlauf, Kmdo. wurde nicht angenommen
    wr_cmd++;
  }
  if (vnc_wr_byte(0x0D)) return 1;		// Fehler berlauf, Kmdo. wurde nicht angenommen
  return 0;
}

// ***********************************************************************
// VNC1L - String empfangen
// Letzte 12 Stellen sind signifikant
// bergabe: wr_string : String (Adresse)
// Rckgabe: status    : 0=ok 1=String nicht vom VNC1L empfangen
// ***********************************************************************
unsigned char vnc_wait_for(char *wr_string) {
unsigned char laenge;			// Stringlnge/Offset
unsigned char treff_index;		// Anzahl der bereinstimmungen
unsigned char buff_index;		// Buffer-Index
unsigned char i;
unsigned char status;
unsigned int rd_answer_versuche;
  rd_answer_versuche = TIMEOUT_KOMMUNIKATION;	// TIMEOUT bis Fehler/Abbruch ggf.
  status       = 0;
  treff_index  = 0;
  buff_index   = 0;
  laenge = strlen(wr_string);
  if (laenge > 12) laenge = 12;	// Lnge begrenzen
  while(rd_answer_versuche && (laenge != treff_index)) {
    status = vnc_rd_byte();
	if (!status) {
	  // neues Zeichen liegt jetzt vor
	  buff_index   = 0;
	  treff_index  = 0;
      for(i = 0; i < laenge; i++) {
		if (vnc.rec_buffer[laenge - i] == *(wr_string + i)) treff_index++;	// bereinstimmung
		buff_index++;
      }
	  rd_answer_versuche = TIMEOUT_KOMMUNIKATION;	// Timeout rcksetzen
	} else rd_answer_versuche--;
  }  
  if (status || (treff_index != laenge)) return 1;	// Fehler, String wurde nicht empfangen
  return 0;	// String wurde empfangen ok
}

// ***********************************************************************
// VNC1L - Auf Antwortstring warten und analysieren
// Die lezten 4 Zeichen der Antwort sind signifikant, ggf. erweitern!)
// bergabe: 1 = Prfung auf Prompt und Fehlermeldungen
//           2 = Prfung nur auf Prompt
// Rckgabe: 1 = Prompt empfangen
//           2 = Command Failed empfangen
//           3 = Bad Command
//           4 = Disk Full
//           5 = Filename Invalid
//           6 = Read Only
//           7 = File Open
//           8 = Dir Not Empty
//           9 = No Disk
//          99 = Timeout
// ***********************************************************************
unsigned char vnc_rd_answer(unsigned char buffer_check) {
unsigned char antw;
unsigned char status;
unsigned int rd_answer_versuche;
  rd_answer_versuche = TIMEOUT_KOMMUNIKATION;	// TIMEOUT bis Fehler/Abbruch ggf. ANPASSEN!
  antw = 0;
  for (status = VNC_BUFFER_SIZE - 1; status; status-- ) vnc.rec_buffer[status] = 0;	// Buffer lschen, status-Variable genutzt
  while(rd_answer_versuche) {
    status = vnc_rd_byte();
	if (!status) {
	  // neues Zeichen liegt jetzt vor
	  switch (buffer_check) {
          case 1: // Command Failed
                  if ((vnc.rec_buffer[3] == 'i') && (vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'd')) antw = 2;
                  // Bad Command
                  if ((vnc.rec_buffer[3] == 'm') && (vnc.rec_buffer[2] == 'a') && (vnc.rec_buffer[1] == 'n') && (vnc.rec_buffer[0] == 'd')) antw = 3;
                  // Disk Full
                  if ((vnc.rec_buffer[3] == 'F') && (vnc.rec_buffer[2] == 'u') && (vnc.rec_buffer[1] == 'l') && (vnc.rec_buffer[0] == 'l')) antw = 4;
                  // Invalid Filename Invalid
                  if ((vnc.rec_buffer[3] == 'a') && (vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'i') && (vnc.rec_buffer[0] == 'd')) antw = 5;
                  // Read Only
                  if ((vnc.rec_buffer[3] == 'O') && (vnc.rec_buffer[2] == 'n') && (vnc.rec_buffer[1] == 'l') && (vnc.rec_buffer[0] == 'y')) antw = 6;
                  // File Open
                  if ((vnc.rec_buffer[3] == 'O') && (vnc.rec_buffer[2] == 'p') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'n')) antw = 7;
                  // Dir Not Empty
                  if ((vnc.rec_buffer[3] == 'm') && (vnc.rec_buffer[2] == 'p') && (vnc.rec_buffer[1] == 't') && (vnc.rec_buffer[0] == 'y')) antw = 8;
                  // No Disk
                  if ((vnc.rec_buffer[3] == 'D') && (vnc.rec_buffer[2] == 'i') && (vnc.rec_buffer[1] == 's') && (vnc.rec_buffer[0] == 'k')) antw = 9;
		  		  // hier knnte auf weitere Fehlermeldungen gecheckt werden...
                  // kein break; hier!!!
          case 2:// nur Prompt Check
                  if ((vnc.rec_buffer[2] == 0x5C) && (vnc.rec_buffer[1] == '>') && (vnc.rec_buffer[0] == 0x0D)) {
                    // Test auf \> und CR war ok
                    antw = 1;
                    //vnc.rec_buffer[0] = 0;	// Buffer verwerfen (letzes Zeichen lschen reicht aus!)
           		  }
      }
      if (antw >= 2) {
	    if(vnc_wr_byte(0x0D)) return 0;	// CR um Fehler zu besttigen, dann kommt wieder ein Prompt, bei Fehler beenden
      }
	  if (antw) break;
	  rd_answer_versuche = TIMEOUT_KOMMUNIKATION;	// Timeout rcksetzen
    } else rd_answer_versuche--;
  }
  if (!antw) antw = 99;	// Timeout
  return antw;
}

// ***********************************************************************
// VNC1L - Auf PROMPT warten
// Bei Fehler Fehler-Nr. zurckgeben
// Rckgabe: 0 = Prompt empfangen
//           xx = Fehler-Nr. der rd_answer Funktion
// ***********************************************************************
unsigned char vnc_prompt_check(void) {
unsigned char status_1, status_2;
  status_1 = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (status_1 == 1) return 0;	// Prompt ohne vorherigen Fehler empfangen, Kmdo. wurde ausgefhrt
  // Fehler aufgetreten, nun nur auf Prompt warten, aber Fehler rckmelden
  status_2 = vnc_rd_answer(PROMPT_CHECK);
  if (status_2 == 1) return status_1;
  return status_2;				// kein Prompt, Timeout
}

// ***********************************************************************
// VNC1L - Erzeugen eines Text-Files auf dem USB-Stick
// maximale Filelnge: 64 kByte
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Textblockes
//           funktion   : File 0=Neu oder 1=Inhalt anhngen
// Rckgabe: status		: Fehler-Nr. / 0=ok
// ***********************************************************************
unsigned char vnc_wr_txtfile(char *filename, char *fileinhalt, char funktion) {
unsigned int vnc_i	= 0;
unsigned int filelen	= 0;
unsigned int filecr	= 0;
char *fileptr;
unsigned char batch_wr	= 0;
unsigned char status	= 0;
  // Byte-Anzahl berechnen
  fileptr = fileinhalt;
  filelen = 0;
  filecr  = 0;
  while (*fileptr != '\a') {	// \a Kennung fr Text-Ende NEU
    if (*fileptr) filelen++; else filecr++;
    fileptr++;
  }
  // 1. Promptzeichen liegt bereits vor
  do {
    if (batch_wr == 4) return 0; // Fertig, ok (letztes Promptzeichen liegt vor)
    switch (batch_wr) {	// Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
      case 1:  // File Position bei Bedarf auf Anfang setzen
               if (!funktion) {
                 if (vnc_wr_cmd("SEK 0")) return 1;			// Fehler berlauf, Kmdo. wurde nicht angenommen
				 break;
               } else batch_wr++;	// Nun wird sofort auch noch case 2: ausgefhrt
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %u",filelen);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               fileptr = fileinhalt;
               for(vnc_i = 0; vnc_i < (filelen + filecr); vnc_i++) {
                 if (*fileptr) {
				   // NULL-Terminierung ausblenden
				   if (vnc_wr_byte(*fileptr)) return 1;		// Fehler berlauf, Byte wurde nicht angenommen
                 }
				 fileptr++;
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
    } // Switch
    batch_wr++;	// nchster Schritt
	status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  } while (status == PROMPT_OK);	// Prfung auf nchstes Promptzeichen >
  return status;	// Fehler
}

// ***********************************************************************
// VNC1L - Einlesen eines Files bestimmter Lnge vom dem USB-Stick in den Speicher
// Lnge des Files muss bekannt sein, ggf. vorher erst mit vnc_rd_dir ermitteln!
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Ziel-Datenspeicheres
//           filelen    : Anzahl der Bytes
// Rckgabe: status		: Fehler-Nr. / 0=ok
// ***********************************************************************
unsigned char vnc_rd_file(char *filename, char *fileinhalt, unsigned int filelen) {
unsigned char batch_wr	= 0;
unsigned char status	= 0;
  // 1. Promptzeichen liegt bereits vor
  if (filelen >= (FILE_BUFFER_SIZE - 1)) {
    return 50;	// Fehler, Buffer zu klein
  }
  do {
    if (batch_wr == 4) return 0; // Fertig, ok (letztes Promptzeichen liegt vor)
    switch (batch_wr) {	// Batchprozess
      case 0:  // File fr lesen ffnen
               sprintf (&vnc.send_buffer[0],"OPR %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               if (vnc_wr_cmd("SEK 0")) return 1;			// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
      case 2:  // File Byteanzahl einlesen
               sprintf (&vnc.send_buffer[0],"RDF %d",filelen);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               while (filelen) {
                 if (vnc_rd_byte()) return 1;				// Fehler, kein Byte empfangen
                 // neues Zeichen liegt vor
                 *fileinhalt = vnc.rec_buffer[0];
                 fileinhalt++;
                 filelen--;
               }
               // Die gelesenen Zeichen stehten nun im Buffer und knnen ausgewertet werden.
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
    } // Switch
    batch_wr++;	// nchster Schritt
	status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  } while (status == PROMPT_OK);	// Prfung auf nchstes Promptzeichen >
  return status;	// Fehler
}

// ***********************************************************************
// VNC1L - Hex-String in Zahl umwandeln
// Beispiel: "$11 $22 $33 $44 D:\>" --> 0x44332211
// maximale Filelnge: 64 kByte (leicht auf 32-Bit erweiterbar)
// bergabe: value   : Adresse des Hex-Strings
// Rckgabe: filelaenge (16 Bit) / 0=Fehler
// ***********************************************************************
unsigned int str_to_int(char *value) {
unsigned char idata temp[4];
unsigned char idata stelle = 0;
unsigned char idata nibble = 0;
unsigned char idata error  = 0;
unsigned char idata wertigkeit = 0;
  for (stelle = 4; stelle; stelle--) {
    if ( (*value == '0') && ( *(value + 1) == '$') ) value += 2;		// 0x ggf. berspringen
    if (*value   == '$') value += 1;									// x berspringen
    wertigkeit = 4;
    temp[stelle - 1] = 0;
    for (nibble = 2; nibble; nibble--) {
      if ((*value >= 'a') && (*value <= 'f')) *value = *value - 0x20;	// in Grobuchstaben umwandeln
      if ((*value >= 'A') && (*value <= 'F')) *value = *value - 0x07;	// Normalisieren
      if ((*value < '0') || (*value > '?')) error = 1; 					// Fehler
      temp[stelle - 1] |= (*value - 0x30) << wertigkeit;
      wertigkeit -= 4;
      value++;
    }
    value++;
  }
  if ((temp[1] != 0) || (temp[0] != 0 )) error = 1; // Fehler > 16-Bit
  if (error) {
    // *(value - 16) = 0x00; // Fehlerrckmeldung ggf. ber Buffer
    temp[2] = 0;
	temp[3] = 0;
  }
 return temp[2] * 256 + temp[3];
}

// ***********************************************************************
// VNC1L - DIR File-Lnge ermitteln
// Zur Zeit bis max. 64 kByte!!!
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Zwischen-Speicheres (nur temporr verwendet)
//           filelen	: Adresse der rckzugebenden Dateilnge
// Rckgabe: status		: Fehler-Nr. / 0=ok
//           Datei-Lnge direkt an Adresse von filelaenge
// ***********************************************************************
unsigned char vnc_rd_dir(char *filename, char *fileinhalt, unsigned int *filelen) {
unsigned char idata status		= 0;
unsigned int idata buff_index		= 0; 
char *anfang;						// Adresse des Hex-Strings (Beginn)
  // 1. Promptzeichen liegt bereits vor
  sprintf (&vnc.send_buffer[0],"DIR %s",filename);
  if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
  // DIR Antwortstring einlesen
  status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  if (status == PROMPT_OK) {
    // Antwortstring liegt jetzt im Buffer (letztes Zeichen ganz vorn)
    for (buff_index = VNC_BUFFER_SIZE; buff_index; buff_index-- ) {
	  *fileinhalt = vnc.rec_buffer[buff_index - 1];	// Inhalt Reihenfolge tauschen
	  if (!(*fileinhalt)) *fileinhalt = ' '; 		// Bufferende-Kennung (0x00) vermeiden
	  fileinhalt++;
    }
	*(fileinhalt) = 0x00; 					// String-Endekennung setzen
	fileinhalt -= (VNC_BUFFER_SIZE - 2);	// Buffer auf Anfang zurck setzen
	// Antwortstring liegt jetzt im Receive-Buffer, Bsp.: "VNCFILE2.TXT $52 $00 $00 $00 D:\>"
    anfang = strchr(fileinhalt,'$');		// Erste Zahl suchen, beginnt mit $: Entspricht LSB der Dateilne
	*filelen = str_to_int(anfang);			// Hex-String in Zahl umwandeln (z.Zt. bis max. 64 kByte!)
	if (!(*filelen)) return 51;				// Fehler, Datei zu lang oder hat Lnge 0
    return 0;								// Ok, Dateilnge gltig
  }
  return status;	// Fehler
}

// ***********************************************************************
// VNC1L - Erzeugen eines Binr-Files auf dem USB-Stick
// (Knnte auch ein Textfile schreiben.)
// maximale Filelnge: 64 kByte
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Datenblockes
//           filelen    : Anzahl der Bytes
// Rckgabe: status		: Fehler-Nr. / 0=ok
// ***********************************************************************
unsigned char vnc_wr_binfile(char *filename, char *fileinhalt, unsigned int filelen) {
unsigned int vnc_i	= 0;
unsigned char batch_wr	= 0;
unsigned char status	= 0;
  if (filelen == 0xFFFF) return 51;	// Datei zu lang
  // 1. Promptzeichen liegt bereits vor
  do {
    if (batch_wr == 4) return 0; // Fertig, ok (letztes Promptzeichen liegt vor)
    switch (batch_wr) {	// Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               if (vnc_wr_cmd("SEK 0")) return 1;			// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               // keine Prfung auf Buffergrenze, ggf. selber erweitern
               for(vnc_i = 0; vnc_i < filelen; vnc_i++) {
                 if (vnc_wr_byte(*fileinhalt)) return 1;	// Fehler berlauf, Byte wurde nicht angenommen
                 fileinhalt++;
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer))  return 1;	// Fehler berlauf, Kmdo. wurde nicht angenommen
               break;
    } // Switch
    batch_wr++;	// nchster Schritt
	status = vnc_rd_answer(PROMPT_UND_ERROR_CHECK);
  } while (status == PROMPT_OK);	// Prfung auf nchstes Promptzeichen >
  return status;	// Fehler
}
