// ********************************************************************
// VNC1L-1A USB-Hostcontroller, USB-Stick am Mikrokontroller verwenden
// 8051-Version / SDCC 2.9.0
// LCD DIP204-4 4x20 (bei Bedarf)
// UART-Monitoring (be Bedarf)
// Version 1.24 / Stand: 16.11.2009
// VNC1L-Interface: SPI (per Software)
// ********************************************************************
// FLASH-Speicherbedarf: ca. 7.2 kByte (inkl. LCD-Funktionen)
//                       ca. 4.5 kByte (ohne LCD-Funktionen)
// XDATA-Speicherbedarf:  ca. 270 Byte (Einsparpotential vorhanden!)
// ********************************************************************
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// Hinweis: Exotische Sticks (z.B. DualCanal OZC) lassen sich vom VNC1L
// offenbar nicht beschreiben.
// Getestete USB-Sticks:
// RUNDISK 512MB FAT16 - ok
// PLATINUM  2GB FAT32 - ok
// CNMEMORY  4GB FAT32 - ok
// TREKSTOR 16GB FAT32 - ok
// OCZ 8GB (RALLY DUAL-CANAL) FAT32 -  kein schreiben mglich
// (VDAP-Firmware kann mit Dual-Canal nicht korrekt umgehen?)
// ********************************************************************
// Aktuelle VDAP Firmware muss auf den VDIP1/VNC1L installiert sein,
// getestet mit Version 3.68. Achtung: Auf neuen VDIP1 ist meist eine
// sehr alte Version drauf, also erstmal ein Firmware-Update machen.
// ********************************************************************
// Das Programm schreibt 2 Dateien auf den USB-Stick. Eine Textdatei
// und eine Binrdatei und zeigt die Verwendung weiterer Kommandos
// bzw. auch das einlesen einer Datei. Ohne LCD sollte die LED nach
// ca. 10s blinken. Wenn ein Fehler auftritt, leuchtet die LED dauerhaft
// und das Programm wird angehalten. Mit LCD werden alle Schritte
// per Monitoring angezeigt. Fr die Inbetriebnahme sehr hilfreich.
// ********************************************************************
//
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// 8051-Typ: AT89C51ED2
// SDCC: --code-loc 0x0000 --iram-size 256 --xram-loc 0x0000 --xram-size 0x0700
//************************************************************************
//
//************************************************************************
// Includebereich
//************************************************************************

#include "at89c51ed2.h"   // Register des AT89C51ED2
#include <string.h>       // Stingfunktionen
#include <stdio.h>		    // Standardfunktion
#include <stdlib.h>         // Standardbibliothek

//************************************************************************
// Definebereich ANPASSEN
//************************************************************************
#define LED_D1_EIN  P0_7 = 0	/* LED */
#define LED_D1_AUS  P0_7 = 1

#define SCLK_L  P1_6 = 0	/* SCLK VNC1L */
#define SCLK_H  P1_6 = 1

#define SDI_L   P1_7 = 0	/* SDI  VNC1L */
#define SDI_H   P1_7 = 1

#define SDO_VNC (P1_5)		/* SDO  VNC1L */

#define CS_VNC_L P1_0 = 0	/* CS   VNC1L */
#define CS_VNC_H P1_0 = 1

#define RE_VNC_L P1_1 = 0	/* RS   VNC1L */
#define RE_VNC_H P1_1 = 1

#define PROMPT_CHECK			2
#define PROMPT_UND_ERROR_CHECK	1
#define PROMPT_OK				1
#define FILE_NEW				0
#define FILE_APPEND				1
#define TIMEOUT_SPI				50		/* Anzahl der SPI Schreib-/Leseversuche */
#define TIMEOUT_READ_WRITE		500		/* Anzahl der Read-/Writeversuche falls Buffer voll */
#define TIMEOUT_KOMMUNIKATION	5		/* Timeout falls keine Kommunikation zu Stande kommt */
//************************************************************************
// Variablenbereich
//************************************************************************
#define VNC_BUFFER_SIZE 60	/* Sende-/Empfangsbuffer Gre mindestens 60 */
struct vnc_struc {
	char  rec_buffer[VNC_BUFFER_SIZE];	// Empfangsbuffer letzte Daten
	char send_buffer[VNC_BUFFER_SIZE];	// Sendebuffer
	unsigned char status;				// Status
};
struct vnc_struc xdata vnc;					// VNC Datensatz 

// Textfile 1 zum testen (Zeigerfeld)
char code *code file1[] =	{
	"USB-Stick am Mikrocontoller -\n\r",
    	"Version: V1.24, Stand: 16.11.2009, Author: Matthias Kahnt\n\r",
	"Text-File 1 vom VNC1L/VDIP1 erzeugt!\n\n\r\a"};	// \a als File-Endekennung NEU

// Textfile 2 zum testen (Zeigerfeld)
char code *code file2[] =	{
	"Text-File 2,\n\r",
	"vom VNC1L auf den Stick geschrieben.\n\r\a" };	// \a als File-Endekennung NEU

// Textfile 3 zum testen (Zeigerfeld)
char code *code file3[] =	{
	"Dieser Text wurde angehngt.\n\r\a" };				// \a als File-Endekennung NEU

// Binrfile zum testen
char code file4[] = {
	0xFA,0x89,0x11,0xFF,0xAA,0x02,0x00,0x1F,
	0x1D,0xBA,0xA6,0x24,0x99,0x2D,0x1E,0x1F,
	0x0D,0x0C,0xA0,0x55,0x30,0x1F,0x16,0xEF,
	0x04,0x22,0x00,0x00,0xCF,0x12,0x5F,0xDF,
	0x69,0x00,0x00,0x1F,0xEF,0x15,0x3F,0xCF,
	0x5F,0x44,0x1F,0x1F,0xBF,0x16,0x13,0x7F,
	0x00,0x10,0x1F,0xDF,0x1F,0x8F,0x16,0xDF,
	0x1F,0x32,0x3F,0x56,0x1F,0x19,0x17,0xFF,
	0x3F,0x42,0xFE,0x46,0x6F,0x99,0x10,0xAF,
	0x6F,0x62,0x36,0x66,0x10,0xAA,0xBB,0xCC };

#define FILE_BUFFER_SIZE	100		/* Mindestens 100 */
char xdata file_buffer[FILE_BUFFER_SIZE];	// Testbuffer fr eingelesenes File
unsigned int file_laenge;			// Byteanzahl der Dateien zu einlesen

#include "delay_51.c" // Pausenfunktionen

//************************************************************************
// USART-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
#define USART_MONITORING	/* aus- / einkommentieren */
#ifdef  USART_MONITORING
	char xdata uart_buffer[25];
//	bit sendactive = 0;
	// Baudrate fest auf 9600 Baud bei 11.0592MHz eingestellt 
	#include "uart_51.c"
#endif

//************************************************************************
// LCD-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
#define LCD_MONITORING /* aus- / einkommentieren */
#ifdef  LCD_MONITORING /* LCD-Funktionen bei Bedarf */

  // **********************************************************************
  // Definebereich
  // **********************************************************************
  #define SPALTEN_MAX  20
  #define ZEILEN_MAX    4
  #define SCROLL_DELAY   1

  // **********************************************************************
 
  #define LCD_DATA_PORT P2
  #define LCD_DATA_READ P2
 
  #define LCD_E1_L P3_5 = 0
  #define LCD_E1_H P3_5 = 1

  #define LCD_RW_L P3_6 = 0
  #define LCD_RW_H P3_6 = 1

  #define LCD_RS_L P3_7 = 0
  #define LCD_RS_H P3_7 = 1

  struct lcd_struc {
    unsigned char x;  // Spalte 
    unsigned char y;  // Zeile
  };
  struct lcd_struc xdata lcd;   // LCD Datensatz 
  char xdata lcd_buffer[25];

  #include "dip204_51.c" // LCD-Basis-Funktionen fr DIP204-4 KS0073
#endif

#include "vnc1l_51.c" // VNC1L-Basis-Funktionen

// ********************************************************************
// System-Initialisierungen ANPASSEN
// ********************************************************************
void init_system (void) {
  // LED
  LED_D1_AUS;
#ifdef USART_MONITORING
  usart_init();
#endif
}

// ***********************************************************************
// Fehlermeldung bewerten
// ***********************************************************************
//	1	= Kommando wurde vom VNC1L nicht akzeptiert
//	2	= Command Failed
//	3	= Bad Command
//	4	= Disk Full
//	5	= Filename Invalid
//	6	= Read Only
//	7	= File Open
//	8	= Dir Not Empty
//	9	= No Disk
//	20	= kein VNC1L mit VDAP-Firmware vorhanden
//	21	= kein USB-Stick angesteckt
//	50	= Buffer zu klein zum Einlesen der Datei
//	51	= Dateilnge > 64kByte
//	99	= Timeout
// ***********************************************************************
void vnc_fehlermeldung(unsigned char fehler_nr, unsigned char fehler_kmdo) {
  LED_D1_EIN;	// LED einschalten, --> Fehler
#ifdef  USART_MONITORING
	sprintf (&uart_buffer[0],"Fehler:%2d / Kmdo.:%2d\n\r",fehler_nr, fehler_kmdo);
	usart_send_string(uart_buffer);
#endif
#ifdef LCD_MONITORING
	sprintf (&lcd_buffer[0],"F-Nr.:%2d (Kmdo.:%2d)",fehler_nr, fehler_kmdo);
	lcd_monitor_string(lcd_buffer); // Fehlermeldung auf LCD anzeigen
#endif
  // Hier Fehler auswerten
  // Zur Zeit ist hier das Programm zu Ende!
  // Je nach schwere des Fehlers spter auch wieder ins Hauptprogramm zurck!
  while(1);
}

// ********************************************************************
// Hauptprogramm
// ********************************************************************
void main (void) {
  init_system();
#ifdef  USART_MONITORING
  usart_send_string("VNC1L: USB-Stick am Mikrocontroller V1.24\n\r");
#endif
#ifdef LCD_MONITORING
  init_lcd();     // LCD-Initialisierung 
  printf_lcd(0,0,"VNC1L-1A -Test-",1); // Einschaltmeldung  
  printf_lcd(0,1,"8051 AT89C51ED2",1);  
  printf_lcd(0,2,"DIP204 (KS0073)",1);
  printf_lcd(0,3,"V1.24 16.11.09 SDCC",1);
  delay_ms(2000);  // Pause
  clear_lcd();      // LCD lschen
#endif

  // VNC1L-Initialisierung
  vnc_init();

  // auf Vorhandensein eines VDIP1/VNC1L mit VDAP-Firmware prfen
  if (vnc_wait_for("VDAP")) vnc_fehlermeldung(20,1);			// Besttigung prfen  
 
   // auf erstes Einstecken des USB-Sticks warten
  if (vnc_wait_for("Device Detected")) vnc_fehlermeldung(21,2);	// Besttigung prfen  

  // auf erstes Prompt warten (Bereitschaft)
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,3);	// Fehlermeldung

  // Extented Command Set
  vnc.status = vnc_wr_cmd("ECS");
  if (vnc.status) vnc_fehlermeldung(1,4);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,4);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

  // Monitor Kommandos Zahlen in ASCII
  vnc.status = vnc_wr_cmd("IPA");
  if (vnc.status) vnc_fehlermeldung(1,5);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,5);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

  // Firmwareversion anzeigen
  vnc.status = vnc_wr_cmd("FWV");
  if (vnc.status) vnc_fehlermeldung(1,6);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,6);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

   // Dir anzeigen
  vnc.status = vnc_wr_cmd("DIR");
  if (vnc.status) vnc_fehlermeldung(1,7);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,7);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

  // Text-File 1 schreiben
  vnc.status = vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_NEW);
  if (vnc.status) vnc_fehlermeldung(vnc.status,8);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt

  // Text-File 2 schreiben
  vnc.status = vnc_wr_txtfile("VNCFILE2.TXT",file2[0],FILE_NEW);
  if (vnc.status) vnc_fehlermeldung(vnc.status,9);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt

  // An Text-File 1 anhngend schreiben
  vnc.status = vnc_wr_txtfile("VNCFILE1.TXT",file3[0],FILE_APPEND);
  if (vnc.status) vnc_fehlermeldung(vnc.status,10);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt

  // Binr-File schreiben
  vnc.status = vnc_wr_binfile("VNCFILE3.BIN",file4,80);
  if (vnc.status) vnc_fehlermeldung(vnc.status,11);	// Fehlermeldung Kommando nicht akzepiert
  // Kommando/Funktion erfolgreich ausgefhrt

  // Datei-Lnge des nun auf den USB-Stick vorhandenen Text-Files 2 ermitteln
  vnc.status = vnc_rd_dir("VNCFILE2.TXT",file_buffer,&file_laenge);
  if (vnc.status) vnc_fehlermeldung(vnc.status,12);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt
  // Lnge steht nun in file_laenge

  if (!vnc.status) {
    // Text-File bekannter Lnge in den Buffer zurcklesen
    vnc.status = vnc_rd_file("VNCFILE2.TXT",file_buffer,file_laenge);
    if (vnc.status) vnc_fehlermeldung(vnc.status,13);	// Fehlermeldung Funktion nicht komplett ausgefhrt
	// Kommando/Funktion erfolgreich ausgefhrt
  }

  // Wenn hier angekommen, ist kein Fehler aufgetreten
  // LED blinkt im Sekundentakt
  // USB-Stick kann entnommen werden
  while(1) {
    LED_D1_EIN;
    delay_ms(1000);
    LED_D1_AUS;
    delay_ms(1000);
  }
}
