// ********************************************************************
// VNC1L-1A USB-Hostcontroller, USB-Stick am Mikrokontroller verwenden
// AVR-Version / GGC 4.3.2
// LCD DIP204-4 4x20 (optional)
// USART-Monitoring (optional)
// Version 1.24 / Stand: 16.11.2009
// AVR-Version / GGC 4.3.2
// Mikrocontroller: ATmega644 (andere mglich)
// Frequenz: mit 1MHz und 8MHz getestet
// VNC1L-Interface: SPI (per Software)
// ********************************************************************
// FLASH-Speicherbedarf: ca. 6.9 kByte (inkl. LCD- und USART Monitoring)
//                       ca. 5.1 kByte (ohne Monitoring)
// SRAM-Speicherbedarf:  ca. 0.8 kByte (Einsparpotential vorhanden!)
// (hauptschlich durch Testdaten, Text usw., PROGMEM nicht verwendet)
// ********************************************************************
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// Hinweis: Exotische Sticks (z.B. DualCanal OZC) lassen sich vom VNC1L
// offenbar nicht beschreiben.
// Getestete USB-Sticks:
// RUNDISK 512MB FAT16 - ok
// PLATINUM  2GB FAT32 - ok
// CNMEMORY  4GB FAT32 - ok
// TREKSTOR 16GB FAT32 - ok
// OCZ 8GB (RALLY DUAL-CANAL) FAT32 -  kein schreiben mglich
// (VDAP-Firmware kann mit Dual-Canal nicht korrekt umgehen?)
// ********************************************************************
// Aktuelle VDAP Firmware muss auf den VDIP1/VNC1L installiert sein,
// getestet mit Version 3.68. Achtung: Auf neuen VDIP1 ist meist eine
// sehr alte Version drauf, also erstmal ein Firmware-Update machen.
// ********************************************************************
// Das Programm schreibt 3 Dateien auf den USB-Stick. Zwei Textdateien
// und eine Binrdatei und zeigt die Verwendung weiterer Kommandos
// bzw. auch das einlesen einer Datei. Ohne LCD sollte die LED nach
// ca. 10s blinken. Wenn ein Fehler auftritt, leuchtet die LED dauerhaft
// und das Programm wird angehalten. Mit LCD/USART werden alle Schritte
// per Monitoring angezeigt. Fr die Inbetriebnahme sehr hilfreich.
// ********************************************************************
//
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// AVR-Typ: ATmega644
//************************************************************************
//
//************************************************************************
// Includebereich
//************************************************************************

#define F_CPU 11059200UL		// FREQUENZ anpassen!
//#define F_CPU 1000000UL		// FREQUENZ anpassen!
//#define F_CPU 8000000UL		// FREQUENZ anpassen!

#include <avr/io.h>         // IO-Grundfunktionen
#include <string.h>		    // Stringfunktion
#include <stdio.h>		    // Standardfunktion
#include <stdlib.h>         // Standardbibliothek
#include <util/delay.h>     // Pasuenfunktionen

//************************************************************************
// Definebereich ANPASSEN
//************************************************************************

// Portverwendung ATmega644
// LED: PB0
// VNC: SCLK PD3
//      SDI  PD4
//      SDO  PD5
//      CS   PD6
//      RS   PD7
//
// optional
// LCD: Data Port A (8 Bit-Interface)
//      E    PB1
//      RW   PB2
//      RS   PB3

#define LED_PIN		0
#define SCLK_PIN	3
#define SDI_PIN		4
#define SDO_PIN		5
#define CS_PIN		6
#define RE_PIN		7

#define LED_PORT	PORTB
#define SPI_PORT	PORTD

#define LED_DDR		DDRB
#define SPI_DDR		DDRD
#define SPI_PIN		PIND

//************************************************************************
//************************************************************************
#define LED_D1_EIN	LED_PORT &= ~(1<<LED_PIN)	/* LED */
#define LED_D1_AUS	LED_PORT |=  (1<<LED_PIN)

#define SCLK_L		SPI_PORT &= ~(1<<SCLK_PIN)		/* SCLK VNC1L */
#define SCLK_H		SPI_PORT |=  (1<<SCLK_PIN)

#define SDI_L		SPI_PORT &= ~(1<<SDI_PIN)		/* SDI  VNC1L */
#define SDI_H		SPI_PORT |=  (1<<SDI_PIN)

#define SDO_VNC		(SPI_PIN & (1<<PD5))			/* SDO  VNC1L */

#define CS_VNC_L	SPI_PORT &= ~(1<<CS_PIN)		/* CS   VNC1L */
#define CS_VNC_H	SPI_PORT |=  (1<<CS_PIN)

#define RE_VNC_L	SPI_PORT &= ~(1<<RE_PIN)		/* RS   VNC1L */
#define RE_VNC_H	SPI_PORT |=  (1<<RE_PIN)

#define PROMPT_CHECK			2
#define PROMPT_UND_ERROR_CHECK	1
#define PROMPT_OK				1
#define FILE_NEW				0
#define FILE_APPEND				1
#define TIMEOUT_SPI				50		/* Anzahl der SPI Schreib-/Leseversuche */
#define TIMEOUT_READ_WRITE		500		/* Anzahl der Read-/Writeversuche falls Buffer voll */
#define TIMEOUT_KOMMUNIKATION	5		/* Timeout falls keine Kommunikation zu Stande kommt */


//************************************************************************
// Variablenbereich
//************************************************************************
#define VNC_BUFFER_SIZE 60	/* Sende-/Empfangsbuffer Gre mindestens 60 */
struct vnc_struc {
	char  rec_buffer[VNC_BUFFER_SIZE];	// Empfangsbuffer letzte Daten
	char send_buffer[VNC_BUFFER_SIZE];	// Sendebuffer
	unsigned char status;				// Status
};
struct vnc_struc vnc;					// VNC Datensatz 

// Textfile 1 zum testen (Zeigerfeld)
char *file1[] =	{
	"USB-Stick am Mikrocontoller -\n\r",
    "Version: V1.24, Stand: 16.11.2009, Author: Matthias Kahnt\n\r",
	"Text-File 1 vom VNC1L/VDIP1 erzeugt!\n\n\r\a"};	// \a als File-Endekennung NEU

// Textfile 2 zum testen (Zeigerfeld)
char *file2[] =	{
	"Text-File 2,\n\r",
	"vom VNC1L auf den Stick geschrieben.\n\r\a" };		// \a als File-Endekennung NEU

// Textfile 3 zum testen (Zeigerfeld)
char *file3[] =	{
	"Dieser Text wurde angehngt.\n\r\a" };				// \a als File-Endekennung NEU

// Binrfile zum testen
char file4[] = {
	0xFA,0x89,0x11,0xFF,0xAA,0x02,0x00,0x1F,
	0x1D,0xBA,0xA6,0x24,0x99,0x2D,0x1E,0x1F,
	0x0D,0x0C,0xA0,0x55,0x30,0x1F,0x16,0xEF,
	0x04,0x22,0x00,0x00,0xCF,0x12,0x5F,0xDF,
	0x69,0x00,0x00,0x1F,0xEF,0x15,0x3F,0xCF,
	0x5F,0x44,0x1F,0x1F,0xBF,0x16,0x13,0x7F,
	0x00,0x10,0x1F,0xDF,0x1F,0x8F,0x16,0xDF,
	0x1F,0x32,0x3F,0x56,0x1F,0x19,0x17,0xFF,
	0x3F,0x42,0xFE,0x46,0x6F,0x99,0x10,0xAF,
	0x6F,0x62,0x36,0x66,0x10,0xAA,0xBB,0xCC };

#define FILE_BUFFER_SIZE	100		/* Mindestens 100 */
char file_buffer[FILE_BUFFER_SIZE];	// Testbuffer fr eingelesenes File
unsigned int file_laenge;			// Byteanzahl der Dateien zu einlesen

//************************************************************************
// USART-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
#define USART_MONITORING	/* aus- / einkommentieren */
#ifdef  USART_MONITORING
	#define  BAUDRATE 9600UL
	char uart_buffer[25];
	#include "usart_avr.c"
#endif

//************************************************************************
// LCD-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
//#define LCD_MONITORING	/* aus- / einkommentieren */
#ifdef  LCD_MONITORING	/* LCD-Funktionen bei Bedarf */

  // **********************************************************************
  // Definebereich
  // **********************************************************************
	#define LCD_E1_PIN			1
	#define LCD_RW_PIN			2
	#define LCD_RS_PIN			3

	#define LCD_CONTROL_PORT	PORTB
	#define LCD_CONTROL_DDR		DDRB

	#define LCD_DATA_PORT		PORTA
	#define LCD_DATA_READ		PINA
	#define LCD_DATA_DDR		DDRA

	#define SPALTEN_MAX			20	/* Anzahl der Zeichen je Zeile */
	#define ZEILEN_MAX			4	/* Anzahl der Zeilen */
	#define SCOLL_DELAY			1	/* Pause zwischen 2 Zeichen in [ms] */

  // **********************************************************************
	#define LCD_E1_L	LCD_CONTROL_PORT &= ~(1<<LCD_E1_PIN)
	#define LCD_E1_H	LCD_CONTROL_PORT |=  (1<<LCD_E1_PIN)

	#define LCD_RW_L	LCD_CONTROL_PORT &= ~(1<<LCD_RW_PIN)
	#define LCD_RW_H	LCD_CONTROL_PORT |=  (1<<LCD_RW_PIN)

	#define LCD_RS_L	LCD_CONTROL_PORT &= ~(1<<LCD_RS_PIN)
	#define LCD_RS_H	LCD_CONTROL_PORT |=  (1<<LCD_RS_PIN)

	struct lcd_struc {
		unsigned char x;	// Spalte 
		unsigned char y;	// Zeile
	};
	struct lcd_struc lcd;	// LCD Datensatz 
	char lcd_buffer[25];
	#include "dip204_avr.c"	// LCD-Basis-Funktionen fr DIP204-4 KS0073
#endif

#include "vnc1l_avr.c" // VNC1L-Basis-Funktionen

// ********************************************************************
// System-Initialisierungen ANPASSEN
// ********************************************************************
void init_system (void) {
  // Portdefinition
  // LED
  LED_DDR |= (1 << LED_PIN);	// Ausgang
  LED_D1_AUS;

  // VNC1L
  SPI_DDR |= ((1 << SCLK_PIN) | (1 << SDI_PIN) | (1 << CS_PIN) | (1 << RE_PIN));	// Ausgnge
  SPI_DDR &= ~(1 << SDO_PIN);	// Eingang

#ifdef USART_MONITORING
	usart_init(BAUDRATE,F_CPU);
#endif

#ifdef LCD_MONITORING  
  // LCD
  LCD_DATA_DDR      = 0xFF;	// Ausgnge
  LCD_CONTROL_DDR  |= ((1 << LCD_E1_PIN) | (1 << LCD_RW_PIN) | (1 << LCD_RS_PIN));	// Ausgnge
#endif
}

// ***********************************************************************
// Fehlermeldung bewerten
// ***********************************************************************
//	1	= Kommando wurde vom VNC1L nicht akzeptiert
//	2	= Command Failed
//	3	= Bad Command
//	4	= Disk Full
//	5	= Filename Invalid
//	6	= Read Only
//	7	= File Open
//	8	= Dir Not Empty
//	9	= No Disk
//	20	= kein VNC1L mit VDAP-Firmware vorhanden
//	21	= kein USB-Stick angesteckt
//	50	= Buffer zu klein zum Einlesen der Datei
//	51	= Dateilnge = 0 oder > 64kByte
//	99	= Timeout
// ***********************************************************************
void vnc_fehlermeldung(unsigned char fehler_nr, unsigned char fehler_kmdo) {
  LED_D1_EIN;	// LED einschalten, --> Fehler
#ifdef  USART_MONITORING
	sprintf (&uart_buffer[0],"Fehler:%2d / Kmdo.:%2d\n\r",fehler_nr, fehler_kmdo);
	usart_send_string(uart_buffer);
#endif
#ifdef LCD_MONITORING
	sprintf (&lcd_buffer[0],"F-Nr.:%2d (Kmdo.:%2d)",fehler_nr, fehler_kmdo);
	lcd_monitor_string(lcd_buffer); // Fehlermeldung auf LCD anzeigen
#endif
  // Hier Fehler auswerten
  // Zur Zeit ist hier das Programm zu Ende!
  // Je nach schwere des Fehlers spter auch wieder ins Hauptprogramm zurck!
  while(1);
}

// ********************************************************************
// Hauptprogramm
// ********************************************************************
int main (void) {
  
  init_system();
#ifdef  USART_MONITORING
	usart_send_string("VNC1L: USB-Stick am Mikrocontroller V1.24\n\r");
#endif
#ifdef LCD_MONITORING
	init_lcd();								// LCD-Initialisierung 
	printf_lcd(0,0,"VNC1L-1A -Test-",1);	// Einschaltmeldung  
	printf_lcd(0,1,"AVR Atmega644",1);  
	printf_lcd(0,2,"DIP204 (KS0073)",1);
	printf_lcd(0,3,"V1.24 16.11.09 GCC",1);
	_delay_ms(2000);						// Pause
	clear_lcd();							// LCD lschen
#endif
  // VNC1L-Initialisierung
  vnc_init();

  // auf Vorhandensein eines VDIP1/VNC1L mit VDAP-Firmware prfen
  if (vnc_wait_for("VDAP")) vnc_fehlermeldung(20,1);			// Besttigung prfen  
 
   // auf erstes Einstecken des USB-Sticks warten
  if (vnc_wait_for("Device Detected")) vnc_fehlermeldung(21,2);	// Besttigung prfen  

  // auf erstes Prompt warten (Bereitschaft)
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,3);	// Fehlermeldung

  // Extented Command Set
  vnc.status = vnc_wr_cmd("ECS");
  if (vnc.status) vnc_fehlermeldung(1,4);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,4);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

  // Monitor Kommandos Zahlen in ASCII
  vnc.status = vnc_wr_cmd("IPA");
  if (vnc.status) vnc_fehlermeldung(1,5);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,5);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

  // Firmwareversion anzeigen
  vnc.status = vnc_wr_cmd("FWV");
  if (vnc.status) vnc_fehlermeldung(1,6);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,6);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

   // Dir anzeigen
  vnc.status = vnc_wr_cmd("DIR");
  if (vnc.status) vnc_fehlermeldung(1,7);			// Fehlermeldung Kommando nicht akzepiert
  vnc.status = vnc_prompt_check();					// Besttigung prfen  
  if (vnc.status) vnc_fehlermeldung(vnc.status,7);	// Fehlermeldung
  // Kommando/Funktion erfolgreich ausgefhrt

  // Text-File 1 schreiben
  vnc.status = vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_NEW);
  if (vnc.status) vnc_fehlermeldung(vnc.status,8);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt

  // Text-File 2 schreiben
  vnc.status = vnc_wr_txtfile("VNCFILE2.TXT",file2[0],FILE_NEW);
  if (vnc.status) vnc_fehlermeldung(vnc.status,9);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt

  // An Text-File 1 anhngend schreiben
  vnc.status = vnc_wr_txtfile("VNCFILE1.TXT",file3[0],FILE_APPEND);
  if (vnc.status) vnc_fehlermeldung(vnc.status,10);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt

  // Binr-File schreiben
  vnc.status = vnc_wr_binfile("VNCFILE3.BIN",file4,80);
  if (vnc.status) vnc_fehlermeldung(vnc.status,11);	// Fehlermeldung Kommando nicht akzepiert
  // Kommando/Funktion erfolgreich ausgefhrt

  // Datei-Lnge des nun auf den USB-Stick vorhandenen Text-Files 2 ermitteln
  vnc.status = vnc_rd_dir("VNCFILE2.TXT",file_buffer,&file_laenge);
  if (vnc.status) vnc_fehlermeldung(vnc.status,12);	// Fehlermeldung Funktion nicht komplett ausgefhrt
  // Kommando/Funktion erfolgreich ausgefhrt
  // Lnge steht nun in file_laenge

  if (!vnc.status) {
    // Text-File bekannter Lnge in den Buffer zurcklesen
    vnc.status = vnc_rd_file("VNCFILE2.TXT",file_buffer,file_laenge);
    if (vnc.status) vnc_fehlermeldung(vnc.status,13);	// Fehlermeldung Funktion nicht komplett ausgefhrt
	// Kommando/Funktion erfolgreich ausgefhrt
  }

  // Wenn hier angekommen, ist kein Fehler aufgetreten
  // LED blinkt im Sekundentakt
  // USB-Stick kann entnommen werden
  while(1) {
    LED_D1_EIN;
    _delay_ms(1000);
    LED_D1_AUS;
    _delay_ms(1000);
  }
}
