// ********************************************************************
// USART - Grundfunktionen 
// AVR-Version / GGC 4.3.2
// Version 1.24 / Stand: 16.11.2009
// Author: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//
// ***********************************************************************
// USART - SFR-Anpassung einiger AVR, ggf. erweitern
// ***********************************************************************
#if defined (__AVR_ATmega128__)
	#define USR		UCSR0A
	#define UCR		UCSR0B
	#define UDR		UDR0
	#define UBRR	UBRR0L
#endif

#if defined (__AVR_ATmega644__)
	#define USR		UCSR0A
	#define UCR		UCSR0B
	#define UBRR	UBRR0L
	#define EICR	EICRB
	#define TXEN	TXEN0
	#define RXEN	RXEN0
	#define RXCIE	RXCIE0
	#define UDR		UDR0
	#define UDRE	UDRE0
#endif

#if defined (__AVR_ATmega32__)
	#define USR		UCSRA
	#define UCR		UCSRB
	#define UBRR	UBRRL
	#define EICR	EICRB
#endif

#if defined (__AVR_ATmega8__)
	#define USR		UCSRA
	#define UCR		UCSRB
	#define UBRR	UBRRL
#endif

#if defined (__AVR_ATmega88__)
	#define USR		UCSR0A
	#define UCR		UCSR0B
	#define UBRR	UBRR0L
	#define TXEN	TXEN0
	#define UDR		UDR0
	#define UDRE	UDRE0
#endif

// ***********************************************************************
// USART - Initialisierung
// bergabe: Baudrate, CPU-Taktfrequenz
// ***********************************************************************
void usart_init(unsigned long baudrate, unsigned long taktfrequenz) {
  // Serielle Schnittstelle, 8n1
  // nur die Sendefunktion freigeben
  UCR = 1 << TXEN;
  // Baudrate einstellen
  // Achtung, mglichst Baudratenquarz einsetzen!
  UBRR = (unsigned char)(taktfrequenz / (baudrate * 16L) - 1);
}

// ***********************************************************************
// USART - Byte senden
// bergabe: zu sendendes Byte
// ***********************************************************************
void usart_send_byte(char send_byte) {
  // Auf Bereitschaft fr nchstes Byte warten
  while(!(USR & (1<<UDRE)));
  // Senden des Bytes
  UDR = send_byte;
}

// ***********************************************************************
// USART - Zeichnkette senden
// bergabe: Adresse der zu sendenden Zeichenkette
// ***********************************************************************
void usart_send_string(char *send_zk) {
  while (*send_zk) usart_send_byte(*send_zk++);
}
