// ********************************************************************
// VNC1L/VDIP1 Vinculum - Grundfunktionen 
// 8051-Version / SDCC 2.9.0
// Stand: V1.2 / 03.11.2009
// Author: Matthias Kahnt
// ********************************************************************
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//
// ***********************************************************************
// VNC1L - USB HOST CONTROLLER - SPI-MODE (MC: Master, VNC1L: SLAVE)
// VDIP1: Jumper-Set: J3 - Pulldown / J4 - Pullup
//        VDAP Firmware muss installiert sein, mit Version 3.68 getestet
// ***********************************************************************

// ***********************************************************************
// VNC1L - Intialisierung SPI-Interface und VNC1L
// SPI Mode
// ***********************************************************************
void vnc_init(void) {
  CS_VNC_L;       // SPI-Interface inaktiv
  SCLK_L;         // Ruhezustand der SPI-Leitungen herstellen
  SDI_H;
  RE_VNC_L;       // Hardware-Reset VNC1L
  delay_ms(100); // Pause
  RE_VNC_H;       // VNC1L starten
  delay_ms(500); // Pause fr Anlauf
}

// ***********************************************************************
// VNC1L - SPI Senden eines Zeichens
// SPI Mode
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: status    : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_spi(unsigned char wr_byte) {
unsigned char stelle, status, cnt;
  cnt = TIMEOUT_SPI; // Anzahl der Schreib-Versuche bis Timeout
  do {
    CS_VNC_H;        // SPI-Interface aktiv
    SDI_H;           // Startbit (immer 1)
    SCLK_H;  SCLK_L; // L/H Flanke
    SDI_L;           // RW-Bit (0=Write)
    SCLK_H;  SCLK_L; // L/H Flanke
    SDI_L;           // ADDR (0=Daten)
    SCLK_H;  SCLK_L; // L/H Flanke
    // 8 Datenbits senden (MSB zuerst)
    for ( stelle = 0x80; stelle; stelle >>= 1 ) {
      if (wr_byte & stelle) SDI_H; else SDI_L;
      SCLK_H;  SCLK_L; // L/H Flanke
    }
    // Statusbit einlesen
    status = SDO_VNC; // Statusbit (0=Accept, 1=Reject)
    SCLK_H;  SCLK_L; // L/H Flanke
    // End Sequenz
    CS_VNC_L;        // SPI-Interface inaktiv
    SCLK_H;  SCLK_L; // L/H Flanke
    SDO_VNC   = 1;   // nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
    cnt--;
	if (status) delay_us(150); // Pause, wenn Byte nicht besttigt wurde
  } while (status && cnt);
#ifdef LCD_MONITORING
  lcd_monitor_byte(wr_byte); // gesendetes Zeichen auf LCD anzeigen
#endif
  return status;
} 

// ***********************************************************************
// VNC1L - SPI Empfangen eines Zeichens
// SPI Mode
// bergabe: rd_fkt    : Funktion (0=Datenbyte, 1=Statusbyte)
// Datenbyte wird im Buffer vnc-Stukturvariable gespeichert
// Rckgabe: status    : 0=Neues Byte im Buffer (ok) 1=kein neues Byte vom VNC1L vorhanden
// ***********************************************************************
unsigned char vnc_rd_spi(unsigned char rd_fkt) {
unsigned char stelle, daten, status;
unsigned int cnt;
  cnt = TIMEOUT_SPI; //  Anzahl der Lese-Versuche bis Timeout
  do {
    daten  = 0;
    CS_VNC_H;        // SPI-Interface aktiv
    SDI_H;           // Startbit (immer 1)
    SCLK_H;  SCLK_L; // L/H Flanke
    SDI_H;           // RW-Bit (1=Read)
    SCLK_H;  SCLK_L; // L/H Flanke
    if (rd_fkt) SDI_H; else SDI_L; // ADDR (0=Daten, 1=SPI-Statusinformation) 
    SCLK_H;  SCLK_L; // L/H Flanke
    // 8 Datenbits empfangen (MSB zuerst)
    for ( stelle = 0x80; stelle; stelle >>= 1 ) {
      if (SDO_VNC) daten |= stelle;
      SCLK_H;  SCLK_L; // L/H Flanke
    }
    // Statusbit einlesen
    status = SDO_VNC; // Statusbit (0=New Data, 1=Old Data)
    SCLK_H;  SCLK_L; // L/H Flanke
    // End Sequenz
    CS_VNC_L;        // SPI-Interface inaktiv
    SCLK_H;  SCLK_L; // L/H Flanke
    SDO_VNC   = 1;   // nur bei 8051: Datenlatch freigeben (8051-Spezifik)  
    if (!status) {
      // empfangenes Zeichen ist ein neues Zeichen
      // Empfangsbuffer um eine Stelle verschieben
      for ( stelle = 15; stelle; stelle-- ) vnc.rec_buffer[stelle] = vnc.rec_buffer[stelle - 1];
      vnc.rec_buffer[0] = daten; // neues Zeichen an 1. Stelle speichern
#ifdef LCD_MONITORING
      if (!rd_fkt) {
	    lcd_monitor_byte(vnc.rec_buffer[0]); // empfangenes Zeichen auf LCD anzeigen
      }
#endif
    }
    cnt--;
	if (status) delay_us(150); // Pause, wenn keine neues Byte vorliegt
  } while (status && cnt);
  return status;
} 

// ***********************************************************************
// VNC1L - Senden eines Zeichens
// bergabe: wr_byte   : Auszugebendes Zeichen
// Rckgabe: status    : 0=Accept (ok) 1=Byte nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_byte(unsigned char wr_byte) {
unsigned char status;
unsigned int cnt;
  cnt = TIMEOUT_READ_WRITE; // Anzahl der Sende-Versuche bis Timeout
  do {
    // VNC1L Status-Byte lesen
    if (vnc_rd_spi(1)) return 1; // Fehler Byte wurde nicht angenommen
	// Statusbyte liegt jetzt im Buffer
	status = vnc.rec_buffer[0] & 0x01; // Test auf VNC1L RECEIVE-BUFFER voll (RXF# Bit0=1)
    cnt--;
	if (status) delay_us(150);  // Pause, wenn RECEIVE-BUFFER voll (RXF#)
  } while (status && cnt);
  if (status || (!cnt)) return 1; // Fehler Byte wurde nicht angenommen
  // VNC1L ist jetzt empfangsbereit
  if (vnc_wr_spi(wr_byte)) return 1; // Fehler Byte wurde nicht angenommen
  return 0; // Byte wurde gesendet
}

// ***********************************************************************
// VNC1L - Empfangen eines Zeichens
// Rckgabe: status    : 0=Neues Byte im Buffer (ok) 1=kein neues Byte vom VNC1L vorhanden
// ***********************************************************************
unsigned char vnc_rd_byte(void) {
unsigned char status;
unsigned int cnt;
  cnt = TIMEOUT_READ_WRITE; // Anzahl der Empfangs-Versuche bis Timeout
  do {
    // VNC1L Status-Byte lesen
    status = vnc_rd_spi(0);       // Wenn Status=1 liegt kein neues Byte vor
	// Prfung auf TXE# im Status-Byte brachte kein Ergebnis
    cnt--;
	if (status) delay_us(150);   // Pause, wenn kein neue Byte vorliegt
  } while (status && cnt);
  if (status || (!cnt)) return 1; // Fehler kein Byte empfangen
  return 0; // Byte wurde empfangen
}

// ***********************************************************************
// VNC1L - Senden eines Kommandos
// bergabe: wr_cmd : Kommando (Adresse)
// Rckgabe: status    : 0=ok 1=Kommando nicht vom VNC1L angenommen
// ***********************************************************************
unsigned char vnc_wr_cmd(char *wr_cmd) {
unsigned char vnc_i; // vnc_offset;
  for(vnc_i = strlen(wr_cmd); vnc_i; vnc_i--) {
    if (vnc_wr_byte(*wr_cmd)) return 1; // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
    wr_cmd++;
  }
  if (vnc_wr_byte(0x0D)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
  return 0;
}

// ***********************************************************************
// VNC1L - String empfangen
// Letzte 12 Stellen sind signifikant
// bergabe: wr_string : String (Adresse)
// Rckgabe: status    : 0=ok 1=String nicht vom VNC1L empfangen
// ***********************************************************************
unsigned char vnc_wait_for(char *wr_string) {
unsigned char laenge;        // Stringlnge/Offset
unsigned char treff_index;   // Anzahl der bereinstimmungen
unsigned char buff_index;    // Buffer-Index
unsigned char i;
unsigned char status;
unsigned int rd_answer_versuche;
  rd_answer_versuche = TIMEOUT_KOMMUNIKATION; // TIMEOUT bis Fehler/Abbruch ggf.
  status       = 0;
  treff_index  = 0;
  buff_index   = 0;
  laenge = strlen(wr_string);
  if (laenge > 12) laenge = 12; // Lnge begrenzen
  while(rd_answer_versuche && (laenge != treff_index)) {
    status = vnc_rd_byte();
	if (!status) {
	  // neues Zeichen liegt jetzt vor
	  buff_index   = 0;
	  treff_index  = 0;
      for(i = 0; i < laenge; i++) {
		if (vnc.rec_buffer[laenge - i] == *(wr_string + i)) treff_index++; // bereinstimmung
		buff_index++;
      }
	  rd_answer_versuche = TIMEOUT_KOMMUNIKATION; // Timeout rcksetzen
	} else rd_answer_versuche--;
  }  
  if (status || (treff_index != laenge)) return 1; // String wurde nicht empfangen
  return 0; // String wurde empfangen ok
}

// ***********************************************************************
// VNC1L - Auf Antwortstring warten und analysieren
// bergabe: 1 = Prfung auf Prompt und Fehlermeldungen
//           2 = Prfung nur auf Prompt
// Rckgabe: 0 = Timeout/Fehler
//           1 = Prompt empfangen
//           2 = command failed empfangen
// ***********************************************************************
unsigned char vnc_rd_answer(unsigned char buffer_check) {
unsigned char antw;
unsigned char status;
unsigned int rd_answer_versuche;
  rd_answer_versuche = TIMEOUT_KOMMUNIKATION; // TIMEOUT bis Fehler/Abbruch ggf. ANPASSEN!
  antw = 0;
  while(rd_answer_versuche) {
    status = vnc_rd_byte();
	if (!status) {
	  // neues Zeichen liegt jetzt vor
	  switch (buffer_check) {
          case 1: // command failed
                  if ((vnc.rec_buffer[2] == 'l') && (vnc.rec_buffer[1] == 'e') && (vnc.rec_buffer[0] == 'd')) antw = 2;
		  		  // hier knnte auf weitere Fehlermeldungen gecheckt werden...
                  // kein break; hier!!!
          case 2: // nur Prompt Check
                  if ((vnc.rec_buffer[2] == 0x5C) && (vnc.rec_buffer[1] == '>') && (vnc.rec_buffer[0] == 0x0D)) {
                    // Test auf \> und CR war ok
                    antw = 1;
                    vnc.rec_buffer[0] = 0; // Buffer verwerfen (letzes Zeichen lschen reicht aus!)
           		  }
      }
      if (antw == 2) {
	    if(vnc_wr_byte(0x0D)) return 0; // CR um Fehler zu besttigen, dann kommt wieder ein Prompt (???), bei Fehler beenden
      }
	  if (antw) break;
	  rd_answer_versuche = TIMEOUT_KOMMUNIKATION; // Timeout rcksetzen
    } else rd_answer_versuche--;
  }
  return antw;
}

// ***********************************************************************
// VNC1L - Auf PROMPT warten
// Bei Fehler Fehlerflag setzen
// Rckgabe: 0 = Prompt empfangen
//           1 = Timeout/Fehler
// ***********************************************************************
unsigned char vnc_prompt_check(void) {
  if(vnc_rd_answer(PROMPT_CHECK)) return 0; else return 1;
}

// ***********************************************************************
// VNC1L - Erzeugen eines Text-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Textblockes
//           funktion   : File 0=Neu oder 1=Inhalt anhngen
// Rckgabe: status    : 0=ok 1=File nicht geschrieben
// ***********************************************************************
unsigned char vnc_wr_txtfile(char *filename, char *fileinhalt, char funktion) {
unsigned int vnc_i = 0;
unsigned int filelen, filecr;
unsigned char *fileptr;
unsigned char batch_wr = 0;
  // Byte-Anzahl berechnen
  fileptr = fileinhalt;
  filelen = 0;
  filecr  = 0;
  while (*fileptr != 0xFF) {
    if (*fileptr) filelen++; else filecr++;
    fileptr++;
  }
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
      case 1:  // File Position bei Bedarf auf Anfang setzen
               if (!funktion) {
                 if (vnc_wr_cmd("SEK 0")) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
				 break;
               } else batch_wr++;  // Nun wird sofort auch noch case 2: ausgefhrt
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               fileptr = fileinhalt;
               for(vnc_i = 0; vnc_i < (filelen + filecr); vnc_i++) {
                 if (*fileptr) {
				   // NULL-Terminierung ausblenden
				   if (vnc_wr_byte(*fileptr)) return 1;  // Fehler berlauf, Byte wurde nicht angenommen!!!
                 }
				 fileptr++;
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
    } // Switch
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return 0; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
  return 0; // nur der Vollstndigkeit halber
}

// ***********************************************************************
// VNC1L - Einlesen eines Files vom dem USB-Stick in den Speicher
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Ziel-Datenspeicheres
//           filelen    : Anzahl der Bytes
// Rckgabe: status    : 0=ok 1=File nicht eingelesen
// ***********************************************************************
unsigned char vnc_rd_file(char *filename, char *fileinhalt, unsigned int filelen) {
unsigned char batch_wr = 0;
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File fr lesen ffnen
               sprintf (&vnc.send_buffer[0],"OPR %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               if (vnc_wr_cmd("SEK 0")) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
      case 2:  // File Byteanzahl einlesen
               sprintf (&vnc.send_buffer[0],"RDF %d",filelen);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               while (filelen) {
                 if (vnc_rd_byte()) return 1;  // Fehler kein Byte empfangen
                 // neues Zeichen liegt vor
                 // neues Zeichen eingelesen (bisher keine Prfung auf Bufferberlauf!!!)
                 *fileinhalt = vnc.rec_buffer[0];
                 fileinhalt++;
                 filelen--;
               }
               // Die gelesenen Zeichen stehten nun im Buffer und knnen ausgewertet werden.
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
    }
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return 0; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
  return 0; // nur der Vollstndigkeit halber  
}

// ***********************************************************************
// VNC1L - Erzeugen eines Binr-Files auf dem USB-Stick
// bergabe: filename   : Adresse des Filenamen
//           fileinhalt : Adresse des Datenblockes
//           filelen    : Anzahl der Bytes
// Rckgabe: status    : 0=ok 1=File nicht geschrieben
// ***********************************************************************
unsigned char vnc_wr_binfile(char *filename, char *fileinhalt, unsigned int filelen) {
unsigned int vnc_i = 0;
unsigned char batch_wr = 0;
  // 1. Promptzeichen liegt bereits vor
  do {
    switch (batch_wr) { // Batchprozess
      case 0:  // File ffnen bzw. Erstellen
               sprintf (&vnc.send_buffer[0],"OPW %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
      case 1:  // File Position immer auf Anfang setzen (knnte man auch auf jede Pos. setzen!)
               if (vnc_wr_cmd("SEK 0")) return 1;  // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
      case 2:  // File Byteanzahl
               sprintf (&vnc.send_buffer[0],"WRF %d",filelen);
               if (vnc_wr_cmd(vnc.send_buffer)) return 1; // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               for(vnc_i = 0; vnc_i < filelen; vnc_i++) {
                 if (vnc_wr_byte(*fileinhalt)) return 1;  // Fehler berlauf, Byte wurde nicht angenommen!!!
                 fileinhalt++;
               }
               break;
      case 3:  // File schlieen
               sprintf (&vnc.send_buffer[0],"CLF %s",filename);
               if (vnc_wr_cmd(vnc.send_buffer))  return 1; // Fehler berlauf, Kmdo. wurde nicht angenommen!!!
               break;
    }
    batch_wr++;    // nchster Schritt
    if (batch_wr == 4) return 0; // Fertig
  } while (vnc_rd_answer(PROMPT_UND_ERROR_CHECK) == PROMPT_OK); // Prfung auf nchstes Promptzeichen >
  return 0; // nur der Vollstndigkeit halber
}

