// ********************************************************************
// VNC1L-1A USB-Hostcontroller, USB-Stick am Mikrokontroller verwenden
// 8051-Version / SDCC 2.9.0
// LCD DIP204-4 4x20 (bei Bedarf)
// Version 1.2 / Stand: 03.11.2009
// VNC1L-Interface: SPI (per Software)
// ********************************************************************
// FLASH-Speicherbedarf: ca. 6.2 kByte (inkl. LCD-Funktionen)
//                       ca. 4.9 kByte (ohne LCD-Funktionen)
// XDATA-Speicherbedarf:  ca. 140 Byte (Einsparpotential vorhanden!)
// ********************************************************************
// Der USB-Stick muss mit FAT oder FAT32 formatiert sein. Es wird nur
// eine primre Partion untersttzt. NTFS Systeme funktionieren nicht.
// Hinweis: Exotische Sticks (z.B. DualCanal OZC) lassen sich vom VNC1L
// offenbar nicht beschreiben.
// Getestete USB-Sticks:
// RUNDISK 512MB FAT16 - ok
// PLATINUM  2GB FAT32 - ok
// CNMEMORY  4GB FAT32 - ok
// TREKSTOR 16GB FAT32 - ok
// OCZ 8GB (RALLY DUAL-CANAL) FAT32 -  kein schreiben mglich
// (VDAP-Firmware kann mit Dual-Canal nicht korrekt umgehen?)
// ********************************************************************
// Aktuelle VDAP Firmware muss auf den VDIP1/VNC1L installiert sein,
// getestet mit Version 3.68. Achtung: Auf neuen VDIP1 ist meist eine
// sehr alte Version drauf, also erstmal ein Firmware-Update machen.
// ********************************************************************
// Das Programm schreibt 2 Dateien auf den USB-Stick. Eine Textdatei
// und eine Binrdatei und zeigt die Verwendung weiterer Kommandos
// bzw. auch das einlesen einer Datei. Ohne LCD sollte die LED nach
// ca. 10s blinken. Wenn ein Fehler auftritt, leuchtet die LED dauerhaft
// und das Programm wird angehalten. Mit LCD werden alle Schritte
// per Monitoring angezeigt. Fr die Inbetriebnahme sehr hilfreich.
// ********************************************************************
//
// Die Programmfunktionen drfen fr private Nutzung verwendet und
// verndert werden. Fr gewerbliche Nutzung ist die Zustimmung
// der Authoren erforderlich.
// Kontakt: Matthias Kahnt, e-mail: az51@gmx.net
// ********************************************************************
// Die Nutzung erfolgt auf eigene Gefahr.
// Fr eventuell entstehende Schden wird keine Haftung bernommen.
// ********************************************************************
//
//************************************************************************
// 8051-Typ: AT89C51ED2
// SDCC: --code-loc 0x0000 --iram-size 256 --xram-loc 0x0000 --xram-size 0x0700
//************************************************************************
//
//************************************************************************
// Includebereich
//************************************************************************

#include "at89c51ed2.h"   // Register des AT89C51ED2
#include <string.h>       // Stingfunktionen
#include <stdio.h>        // Standardfunktionen

//************************************************************************
// Definebereich ANPASSEN
//************************************************************************
#define LED_D1_EIN  P0_7 = 0	/* LED */
#define LED_D1_AUS  P0_7 = 1

#define SCLK_L  P1_6 = 0	/* SCLK VNC1L */
#define SCLK_H  P1_6 = 1

#define SDI_L   P1_7 = 0	/* SDI  VNC1L */
#define SDI_H   P1_7 = 1

#define SDO_VNC (P1_5)		/* SDO  VNC1L */

#define CS_VNC_L P1_0 = 0	/* CS   VNC1L */
#define CS_VNC_H P1_0 = 1

#define RE_VNC_L P1_1 = 0	/* RS   VNC1L */
#define RE_VNC_H P1_1 = 1

#define X2_MODE
#define PROMPT_CHECK           2
#define PROMPT_UND_ERROR_CHECK 1
#define PROMPT_OK              1
#define FILE_NEW               0
#define FILE_APPEND            1
#define TIMEOUT_SPI            50    /* Anzahl der SPI Schreib-/Leseversuche */
#define TIMEOUT_READ_WRITE     500   /* Anzahl der Read-/Writeversuche falls Buffer voll */
#define TIMEOUT_KOMMUNIKATION  5     /* Timeout falls keine Kommunikation zu Stande kommt */

//************************************************************************
// Variablenbereich
//************************************************************************
unsigned char batch = 0;     // Batchprozess in Main-Funktion
struct vnc_struc {
  char  rec_buffer[20];      // Empfangsbuffer letzte Daten
  char send_buffer[20];      // Sendebuffer
};
struct vnc_struc xdata vnc;        // VNC Datensatz 

// Textfile zum testen (Zeigerfeld)
char code txt_z1[]  = "Systemkonfiguration\x00D\x00A";
char code txt_z2[]  = "V1.2 vom 03.11.2009\x00D\x00A";
char code txt_z3[]  = "Device: AT89C51ED2\x00D\x00A";
char code txt_z4[]  = "Frequenz: 11.0592MHz\x00D\x00A";
char code txt_z5[]  = "Hardware: PRO-51 Board\x00D\x00A";
char code txt_z6[]  = "USB Hst: VDIP1 VNC1L\x00D\x00A";
char code txt_z7[]  = "Firmware: 03.68 VDAP\x00D\x00A";
char code txt_z8[]  = "Anzeige: DIP204 4x20\x00D\x00A";
char code txt_z9[]  = "Compiler: SDCC 2.9.0\x00D\x00A";
char code txt_z10[] = "Prg.: FLIP\x00D\x00A";
char code txt_z11[] = "\x0FF"; // File-Endekennung
char code * code file1[] = {
	txt_z1,	txt_z2, txt_z3,	txt_z4,	 txt_z5, txt_z6,
	txt_z7,	txt_z8,	txt_z9,	txt_z10, txt_z11 };

// Binrfile zum testen
char code file2[] = {
  0xFA,0x89,0x11,0x23,0xAA,0x02,0x00,0x1F,
  0x1D,0xBA,0xA6,0x24,0x99,0x2D,0x1E,0x1F,
  0x0D,0x0C,0xA0,0x55,0x30,0x1F,0x16,0xEF,
  0x04,0x22,0x00,0x00,0xCF,0x12,0x5F,0xDF,
  0x69,0x00,0x00,0x1F,0xEF,0x15,0x3F,0xCF,
  0x5F,0x44,0x1F,0x1F,0xBF,0x16,0x13,0x7F,
  0x00,0x10,0x1F,0xDF,0x1F,0x8F,0x16,0xDF,
  0x1F,0x32,0x3F,0x56,0x1F,0x19,0x17,0xFF,
  0x3F,0x42,0xFE,0x46,0x6F,0x99,0x10,0xAF,
  0x6F,0x62,0x36,0x66,0x10,0xAA,0xBB,0xCC };

char xdata test_buffer[100]; // Testbuffer fr eingelesenes File, Lnge wird nicht geprft!!!

#include "delay_51.c" // Pausenfunktionen

//************************************************************************
// LCD-Monitoring (bei Bedarf) ANPASSEN
//************************************************************************
#define LCD_MONITORING /* aus- / einkommentieren */
#ifdef  LCD_MONITORING /* LCD-Funktionen bei Bedarf */

  // **********************************************************************
  // Definebereich
  // **********************************************************************
  #define SPALTEN_MAX  20
  #define ZEILEN_MAX    4
  #define SCROLL_DELAY   1

  // **********************************************************************
 
  #define LCD_DATA_PORT P2
  #define LCD_DATA_READ P2
 
  #define LCD_E1_L P3_5 = 0
  #define LCD_E1_H P3_5 = 1

  #define LCD_RW_L P3_6 = 0
  #define LCD_RW_H P3_6 = 1

  #define LCD_RS_L P3_7 = 0
  #define LCD_RS_H P3_7 = 1

  struct lcd_struc {
    unsigned char x;  // Spalte 
    unsigned char y;  // Zeile
  };
  struct lcd_struc idata lcd;   // LCD Datensatz 
  char idata lcd_buffer[25];

  #include "dip204_51.c" // LCD-Basis-Funktionen fr DIP204-4 KS0073
#endif

#include "vnc1l_51.c" // VNC1L-Basis-Funktionen

// ********************************************************************
// System-Initialisierungen ANPASSEN
// ********************************************************************
void init_system (void) {
  // LED
  LED_D1_AUS;
}

// ***********************************************************************
// Fehlermeldung
// ***********************************************************************
void vnc_fehlermeldung(unsigned char fehler_nr) {
  LED_D1_EIN;
#ifdef LCD_MONITORING
  sprintf (&lcd_buffer[0],"->Fehler-Nr.: %3d",fehler_nr);
  lcd_monitor_string(lcd_buffer); // Fehlermeldung auf LCD anzeigen
#endif
  // Hier Fehler auswerten
  // Zur Zeit ist hier das Programm zu Ende!
  while(1);
}

// ********************************************************************
// Hauptprogramm
// ********************************************************************
void main (void) {
  init_system();
#ifdef LCD_MONITORING
  init_lcd();     // LCD-Initialisierung 
  printf_lcd(0,0,"VNC1L-1A -Test-",1); // Einschaltmeldung  
  printf_lcd(0,1,"8051 AT89C51ED2",1);  
  printf_lcd(0,2,"DIP204 (KS0073)",1);
  printf_lcd(0,3,"V1.2 03.11.2009 SDCC",1);
  delay_ms(2000);  // Pause
  clear_lcd();      // LCD lschen
#endif

  vnc_init();       // VNC1L-Initialisierung

  // auf Vorhandensein eines VDIP1/VNC1L mit VDAP-Firmware prfen
  if (vnc_wait_for("VDAP")) vnc_fehlermeldung(4); // Besttigung prfen  
 
   // auf erstes Einstecken des USB-Sticks warten
  if (vnc_wait_for("Device Detected")) vnc_fehlermeldung(5); // Besttigung prfen  

  // auf erstes Prompt warten (Bereitschaft)
  if (vnc_prompt_check()) vnc_fehlermeldung(10); // Besttigung prfen  

  // Extented Command Set
  if (vnc_wr_cmd("ECS"))  vnc_fehlermeldung(11);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

  // Monitor Kommandos in ASCII
  if (vnc_wr_cmd("IPA"))  vnc_fehlermeldung(12);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

  // Firmwareversion anzeigen
  if (vnc_wr_cmd("FWV"))  vnc_fehlermeldung(13);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

   // Dir anzeigen
  if (vnc_wr_cmd("DIR"))  vnc_fehlermeldung(14);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

  // Text-File schreiben
  if (vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_NEW)) vnc_fehlermeldung(20);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

  // Binr-File schreiben
  if (vnc_wr_binfile("VNCFILE2.BIN",file2,80)) vnc_fehlermeldung(21);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

  // Text-File (Teil) einlesen
  if (vnc_rd_file("VNCFILE1.TXT",test_buffer,40)) vnc_fehlermeldung(22);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen  

  // Text-File anhngend schreiben
  if (vnc_wr_txtfile("VNCFILE1.TXT",file1[0],FILE_APPEND)) vnc_fehlermeldung(30);
  if (vnc_prompt_check()) vnc_fehlermeldung(99); // Besttigung prfen

  // kein Fehler aufgetreten --> die LED blinkt
  // USB-Stick kann entnommen werden
  while(1) {
    LED_D1_EIN;
    delay_ms(1000);
    LED_D1_AUS;
    delay_ms(1000);
  }
}
